@@ -23,7 +23,9 @@
 // Copyright 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by the LiMux project of the city of Munich
 // Copyright 2018 Chinmoy Ranjan Pradhan <chinmoyrp65@protonmail.com>
 // Copyright 2018 Adam Reichold <adam.reichold@t-online.de>
-// Copyright 2018 Nelson Benítez León <nbenitezl@gmail.com>
+// Copyright 2018, 2019 Nelson Benítez León <nbenitezl@gmail.com>
+// Copyright 2019 Oliver Sander <oliver.sander@tu-dresden.de>
+// Copyright 2019 Tomoyuki Kubota <himajin100000@gmail.com>
 //
 //========================================================================
 
@@ -63,8 +65,8 @@ char* pdfDocEncodingToUTF16 (const GooString* orig, int* length)
   char *result = new char[(*length)];
   const char *cstring = orig->c_str();
   //unicode marker
-  result[0] = (char)0xfe;
-  result[1] = (char)0xff;
+  result[0] = '\xfe';
+  result[1] = '\xff';
   //convert to utf16
   for(int i=2,j=0; i<(*length); i+=2,j++) {
     Unicode u = pdfDocEncoding[(unsigned int)((unsigned char)cstring[j])]&0xffff;
@@ -614,7 +616,7 @@ FormField::FormField(PDFDoc *docA, Object &&aobj, const Ref aref, FormField *par
   xref = doc->getXRef();
   obj = std::move(aobj);
   Dict* dict = obj.getDict();
-  ref.num = ref.gen = 0;
+  ref = aref;
   type = ty;
   parent = parentA;
   numChildren = 0;
@@ -627,26 +629,23 @@ FormField::FormField(PDFDoc *docA, Object &&aobj, const Ref aref, FormField *par
   quadding = quaddingLeftJustified;
   hasQuadding = false;
 
-  ref = aref;
-
   //childs
   Object obj1 = dict->lookup("Kids");
   if (obj1.isArray()) {
     // Load children
     for (int i = 0 ; i < obj1.arrayGetLength(); i++) {
-      const Object &childRef = obj1.arrayGetNF(i);
-      if (!childRef.isRef()) {
+      Ref childRef;
+      Object childObj = obj1.getArray()->get(i, &childRef);
+      if (childRef == Ref::INVALID()) {
         error (errSyntaxError, -1, "Invalid form field renference");
         continue;
       }
-      Object childObj = obj1.arrayGet(i);
       if (!childObj.isDict()) {
         error (errSyntaxError, -1, "Form field child is not a dictionary");
         continue;
       }
 
-      const Ref ref = childRef.getRef();
-      if (usedParents->find(ref.num) == usedParents->end()) {
+      if (usedParents->find(childRef.num) == usedParents->end()) {
         // Field child: it could be a form field or a widget or composed dict
         const Object &objParent = childObj.dictLookupNF("Parent");
 	Object obj3 = childObj.dictLookup("Parent");
@@ -655,7 +654,7 @@ FormField::FormField(PDFDoc *docA, Object &&aobj, const Ref aref, FormField *par
           // We create the field, if it's composed
           // it will create the widget as a child
           std::set<int> usedParentsAux = *usedParents;
-          usedParentsAux.insert(ref.num);
+          usedParentsAux.insert(childRef.num);
 
           if (terminal) {
             error(errSyntaxWarning, -1, "Field can't have both Widget AND Field as kids\n");
@@ -664,7 +663,7 @@ FormField::FormField(PDFDoc *docA, Object &&aobj, const Ref aref, FormField *par
 
           numChildren++;
           children = (FormField**)greallocn(children, numChildren, sizeof(FormField*));
-          children[numChildren - 1] = Form::createFieldFromDict(std::move(childObj), doc, ref, this, &usedParentsAux);
+          children[numChildren - 1] = Form::createFieldFromDict(std::move(childObj), doc, childRef, this, &usedParentsAux);
         } else {
 	  Object obj2 = childObj.dictLookup("Subtype");
 	  if (obj2.isName("Widget")) {
@@ -673,7 +672,7 @@ FormField::FormField(PDFDoc *docA, Object &&aobj, const Ref aref, FormField *par
 	      error(errSyntaxWarning, -1, "Field can't have both Widget AND Field as kids\n");
 	      continue;
 	    }
-	    _createWidget(&childObj, ref);
+	    _createWidget(&childObj, childRef);
 	  }
 	}
       }
@@ -829,8 +828,7 @@ FormWidget* FormField::findWidgetByRef (Ref aref)
 {
   if (terminal) {
     for(int i=0; i<numChildren; i++) {
-      if (widgets[i]->getRef().num == aref.num 
-          && widgets[i]->getRef().gen == aref.gen)
+      if (widgets[i]->getRef() == aref)
         return widgets[i];
     }
   } else {
@@ -994,7 +992,10 @@ FormFieldButton::FormFieldButton(PDFDoc *docA, Object &&aobj, const Ref ref, For
     } 
   }
 
-  if (btype != formButtonPush) {
+  bool isChildRadiobutton = btype == formButtonRadio && terminal && parent && parent->getType() == formButton;
+  // Ignore "V" for child radiobuttons, so FormFieldButton::getState() does not use it and instead uses the
+  // "V" of the parent, which is the real value indicating the active field in the radio group. Issue #159
+  if (btype != formButtonPush && !isChildRadiobutton) {
     // Even though V is inheritable we are interested in the value of this
     // field, if not present it's probably because it's a button in a set.
     appearanceState = dict->lookup("V");
@@ -1213,49 +1214,58 @@ FormFieldText::~FormFieldText()
 
 double FormFieldText::getTextFontSize()
 {
-  GooList* daToks = new GooList();
+  std::vector<GooString*>* daToks = new std::vector<GooString*>();
   int idx = parseDA(daToks);
   double fontSize = -1;
   if (idx >= 0) {
     char* p = nullptr;
-    fontSize = strtod(static_cast<GooString*>(daToks->get(idx))->c_str(), &p);
+    fontSize = strtod((*daToks)[idx]->c_str(), &p);
     if (!p || *p)
       fontSize = -1;
   }
-  deleteGooList<GooString>(daToks);
+  for (auto entry : *daToks) {
+    delete entry;
+  }
+  delete daToks;
   return fontSize;
 }
 
 void FormFieldText::setTextFontSize(int fontSize)
 {
   if (fontSize > 0 && obj.isDict()) {
-    GooList* daToks = new GooList();
+    std::vector<GooString*>* daToks = new std::vector<GooString*>();
     int idx = parseDA(daToks);
     if (idx == -1) {
       error(errSyntaxError, -1, "FormFieldText:: invalid DA object\n");
-      deleteGooList<GooString>(daToks);
+      for (auto entry : *daToks) {
+        delete entry;
+      }
+      delete daToks;
       return;
     }
     if (defaultAppearance)
       delete defaultAppearance;
     defaultAppearance = new GooString;
-    for (int i = 0; i < daToks->getLength(); ++i) {
+    for (std::size_t i = 0; i < daToks->size(); ++i) {
       if (i > 0)
         defaultAppearance->append(' ');
-      if (i == idx) {
+      if (i == (std::size_t)idx) {
         defaultAppearance->appendf("{0:d}", fontSize);
       } else {
-        defaultAppearance->append(static_cast<GooString*>(daToks->get(i)));
+        defaultAppearance->append((*daToks)[i]);
       }
     }
-    deleteGooList<GooString>(daToks);
+    for (auto entry : *daToks) {
+      delete entry;
+    }
+    delete daToks;
     obj.dictSet("DA", Object(defaultAppearance->copy()));
     xref->setModifiedObject(&obj, ref);
     updateChildrenAppearance();
   }
 }
 
-int FormFieldText::tokenizeDA(const GooString* da, GooList* daToks, const char* searchTok)
+int FormFieldText::tokenizeDA(const GooString* da, std::vector<GooString*>* daToks, const char* searchTok)
 {
   int idx = -1;
   if(da && daToks) {
@@ -1270,8 +1280,8 @@ int FormFieldText::tokenizeDA(const GooString* da, GooList* daToks, const char*
         }
         GooString* tok = new GooString(da, i, j - i);
         if (searchTok && !tok->cmp(searchTok))
-          idx = daToks->getLength();
-        daToks->push_back(tok);
+          idx = daToks->size();
+	daToks->push_back(tok);
         i = j;
       }
     }
@@ -1279,7 +1289,7 @@ int FormFieldText::tokenizeDA(const GooString* da, GooList* daToks, const char*
   return idx;
 }
 
-int FormFieldText::parseDA(GooList* daToks)
+int FormFieldText::parseDA(std::vector<GooString*>* daToks)
 {
   int idx = -1;
   if (obj.isDict()) {
