@@ -6,6 +6,8 @@
 class TestObject : public QObject
 {
     Q_OBJECT
+public:
+    TestObject(QObject *parent = nullptr) : QObject(parent) { }
 private slots:
     void benchDefaultConstructor();
     void benchMoveConstructor();
