@@ -24,6 +24,8 @@
 // Copyright (C) 2015, 2018 Adam Reichold <adamreichold@myopera.com>
 // Copyright (C) 2016 Kenji Uno <ku@digitaldolphins.jp>
 // Copyright (C) 2018 Martin Packman <gzlist@googlemail.com>
+// Copyright (C) 2019 Christian Persch <chpe@src.gnome.org>
+// Copyright (C) 2019 Oliver Sander <oliver.sander@tu-dresden.de>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -36,6 +38,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <limits.h>
+#include "goo/gfile.h"
 #include "goo/gmem.h"
 #include "SplashErrorCodes.h"
 #include "SplashBitmap.h"
@@ -44,7 +47,6 @@
 #include "goo/PNGWriter.h"
 #include "goo/TiffWriter.h"
 #include "goo/ImgWriter.h"
-#include "goo/GooList.h"
 
 //------------------------------------------------------------------------
 // SplashBitmap
@@ -52,7 +54,7 @@
 
 SplashBitmap::SplashBitmap(int widthA, int heightA, int rowPadA,
 			   SplashColorMode modeA, bool alphaA,
-			   bool topDown, GooList *separationListA) {
+			   bool topDown, std::vector<GfxSeparationColorSpace*> *separationListA) {
   width = widthA;
   height = heightA;
   mode = modeA;
@@ -115,24 +117,24 @@ SplashBitmap::SplashBitmap(int widthA, int heightA, int rowPadA,
       rowSize = -rowSize;
     }
     if (alphaA) {
-      alpha = (Guchar *)gmallocn(width, height);
+      alpha = (unsigned char *)gmallocn(width, height);
     } else {
       alpha = nullptr;
     }
   } else {
     alpha = nullptr;
   }
-  separationList = new GooList();
+  separationList = new std::vector<GfxSeparationColorSpace*>();
   if (separationListA != nullptr)
-    for (int i = 0; i < separationListA->getLength(); i++)
-      separationList->push_back(((GfxSeparationColorSpace *) separationListA->get(i))->copy());
+    for (std::size_t i = 0; i < separationListA->size(); i++)
+      separationList->push_back((GfxSeparationColorSpace*)( (*separationListA)[i])->copy());
 }
 
 SplashBitmap *SplashBitmap::copy(SplashBitmap *src) {
   SplashBitmap *result = new SplashBitmap(src->getWidth(), src->getHeight(), src->getRowPad(), 
     src->getMode(), src->getAlphaPtr() != nullptr, src->getRowSize() >= 0, src->getSeparationList());
-  Guchar *dataSource = src->getDataPtr();
-  Guchar *dataDest = result->getDataPtr();
+  unsigned char *dataSource = src->getDataPtr();
+  unsigned char *dataDest = result->getDataPtr();
   int amount = src->getRowSize();
   if (amount < 0) {
     dataSource = dataSource + (src->getHeight() - 1) * amount;
@@ -157,7 +159,10 @@ SplashBitmap::~SplashBitmap() {
     }
   }
   gfree(alpha);
-  deleteGooList<GfxSeparationColorSpace>(separationList);
+  for (auto entry : *separationList) {
+    delete entry;
+  }
+  delete separationList;
 }
 
 
@@ -165,7 +170,7 @@ SplashError SplashBitmap::writePNMFile(char *fileName) {
   FILE *f;
   SplashError e;
 
-  if (!(f = fopen(fileName, "wb"))) {
+  if (!(f = openFile(fileName, "wb"))) {
     return splashErrOpenFile;
   }
 
@@ -262,7 +267,7 @@ SplashError SplashBitmap::writeAlphaPGMFile(char *fileName) {
   if (!alpha) {
     return splashErrModeMismatch;
   }
-  if (!(f = fopen(fileName, "wb"))) {
+  if (!(f = openFile(fileName, "wb"))) {
     return splashErrOpenFile;
   }
   fprintf(f, "P5\n%d %d\n255\n", width, height);
@@ -322,7 +327,7 @@ void SplashBitmap::getPixel(int x, int y, SplashColorPtr pixel) {
   }
 }
 
-Guchar SplashBitmap::getAlpha(int x, int y) {
+unsigned char SplashBitmap::getAlpha(int x, int y) {
   return alpha[y * width + x];
 }
 
@@ -338,7 +343,7 @@ SplashError SplashBitmap::writeImgFile(SplashImageFileFormat format, const char
   FILE *f;
   SplashError e;
 
-  if (!(f = fopen(fileName, "wb"))) {
+  if (!(f = openFile(fileName, "wb"))) {
     return splashErrOpenFile;
   }
 
@@ -538,8 +543,8 @@ void SplashBitmap::getXBGRLine(int yl, SplashColorPtr line, ConversionMode conve
   }
 }
 
-static inline Guchar div255(int x) {
-  return (Guchar)((x + (x >> 8) + 0x80) >> 8);
+static inline unsigned char div255(int x) {
+  return (unsigned char)((x + (x >> 8) + 0x80) >> 8);
 }
 
 bool SplashBitmap::convertToXBGR(ConversionMode conversionMode) {
@@ -549,11 +554,11 @@ bool SplashBitmap::convertToXBGR(ConversionMode conversionMode) {
       const SplashColorPtr dbegin = data;
       const SplashColorPtr dend = data + rowSize * height;
 
-      Guchar *const abegin = alpha;
-      Guchar *const aend = alpha + width * height;
+      unsigned char *const abegin = alpha;
+      unsigned char *const aend = alpha + width * height;
 
       SplashColorPtr d = dbegin;
-      Guchar *a = abegin;
+      unsigned char *a = abegin;
 
       if (conversionMode == conversionAlphaPremultiplied) {
           for (; d < dend && a < aend; d += 4, a += 1) {
