@@ -6,11 +6,12 @@
 //
 // Copyright 2015 André Guerreiro <aguerreiro1985@gmail.com>
 // Copyright 2015 André Esser <bepandre@hotmail.com>
-// Copyright 2015, 2017, 2018 Albert Astals Cid <aacid@kde.org>
+// Copyright 2015, 2017-2019 Albert Astals Cid <aacid@kde.org>
 // Copyright 2016 Markus Kilås <digital@markuspage.com>
-// Copyright 2017 Hans-Ulrich Jüttner <huj@froreich-bioscientia.de>
-// Copyright 2017 Adrian Johnson <ajohnson@redneon.com>
+// Copyright 2017, 2019 Hans-Ulrich Jüttner <huj@froreich-bioscientia.de>
+// Copyright 2017, 2019 Adrian Johnson <ajohnson@redneon.com>
 // Copyright 2018 Chinmoy Ranjan Pradhan <chinmoyrp65@protonmail.com>
+// Copyright 2019 Alexey Pavlov <alexpux@gmail.com>
 //
 //========================================================================
 
@@ -23,18 +24,20 @@
 #include <time.h>
 #include <hasht.h>
 #include <fstream>
-#include <libgen.h>
 #include "parseargs.h"
 #include "Object.h"
 #include "Array.h"
+#include "goo/gbasename.h"
 #include "Page.h"
 #include "PDFDoc.h"
 #include "PDFDocFactory.h"
 #include "Error.h"
 #include "GlobalParams.h"
+#include "SignatureHandler.h"
 #include "SignatureInfo.h"
 #include "Win32Console.h"
 #include "numberofcharacters.h"
+#include <libgen.h>
 
 static const char * getReadableSigState(SignatureValidationStatus sig_vs)
 {
@@ -105,9 +108,7 @@ static void dumpSignature(int sig_num, int sigCount, FormWidgetSignature *sig_wi
     // since { is the magic character to replace things we need to put it twice where
     // we don't want it to be replaced
     GooString *format = GooString::format("{{0:s}}.sig{{1:{0:d}d}}", sigCountLength);
-    char *filenameCopy = strdup(filename);
-    GooString *path = GooString::format(format->c_str(), basename(filenameCopy), sig_num);
-    free(filenameCopy);
+    GooString *path = GooString::format(format->c_str(), gbasename(filename).c_str(), sig_num);
     printf("Signature #%d (%u bytes) => %s\n", sig_num, signature->getLength(), path->c_str());
     std::ofstream outfile(path->c_str(), std::ofstream::binary);
     outfile.write(signature->c_str(), signature->getLength());
@@ -116,12 +117,15 @@ static void dumpSignature(int sig_num, int sigCount, FormWidgetSignature *sig_wi
     delete path;
 }
 
+static GooString nssDir;
 static bool printVersion = false;
 static bool printHelp = false;
 static bool dontVerifyCert = false;
 static bool dumpSignatures = false;
 
 static const ArgDesc argDesc[] = {
+  {"-nssdir", argGooString, &nssDir,     0,
+   "path to directory of libnss3 database"},
   {"-nocert", argFlag,     &dontVerifyCert,     0,
    "don't perform certificate validation"},
   {"-dump",   argFlag,     &dumpSignatures,     0,
@@ -169,6 +173,8 @@ int main(int argc, char *argv[])
 
   fileName = new GooString(argv[argc - 1]);
 
+  SignatureHandler::setNSSDir(nssDir);
+
   // open PDF file
   doc = PDFDocFactory().createPDFDoc(*fileName, nullptr, nullptr);
 
@@ -270,9 +276,9 @@ int main(int argc, char *argv[])
   exitCode = 0;
 
 end:
-  delete globalParams;
   delete fileName;
   delete doc;
+  delete globalParams;
 
   return exitCode;
 }
